''*********************************************************************
''
''   File....... FDSQ_Test.spin
''   Purpose.... Test/demo for the four port full duplex object
''   Author..... Kurt Winn - Copyright (c) 2012
''   Updated.... 2015 added rxstr test for serial objects
''   E-mail..... kurt.winn@yahoo.com
''
''   Tests FourPortSerial objects by sharing the tx pin of one port
''   with the rx pin of another port. The sequence is as follows:
''   TX1 > RX2 P1, TX2 > RX3 P2, TX3 > RX1 P0
''   Port 0 is used for PST
''  
''*********************************************************************

con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                  ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                 ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                           ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                       ' ticks in 1us

  nu = -1                                               ' not used                         
  
con { io pins }

  rx0 = 31                                                       ' port 0 programming / terminal
  tx0 = 30
  rx1 = 0                                                       ' port 1
  tx1 = 1
  rx2 = 1                                                       ' port2
  tx2 = 2
  rx3 = 2                                                       ' port3
  tx3 = 0

  SDA = 29                                                       ' eeprom / i2c
  SCL = 28

con { pst formatting }

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR
  #14, GOTOX, GOTOY, CLS


obj

  term : "FDSQ-64"
'  term : "FDSQ-128"
'  term : "FDSQ-V"

var

  byte count

dat { test string }

  RawInput              byte "Debugging using PST on port 0  ", 0       
pub main | port 
  pause(1000)
  term.Init                                             

' AddPort(port,rxpin,txpin,ctspin,rtspin,rtsthreshold,mode,baudrate) Add up to four ports - nu if pin not used
  pause(1000)
  term.AddPort(0,rx0,tx0,nu,nu,0,0,115200)                ' PST connection
  term.AddPort(1,rx1,tx1,nu,nu,16,0,115200)                 ' receive on pin 0 send on pin 1
  term.AddPort(2,rx2,tx2,nu,nu,16,0,115200)                 ' receive on pin 1 send on pin 2
  term.AddPort(3,rx3,tx3,nu,nu,16,0,115200)                 ' receive on pin 2 send on pin 0
    
  pause(1000)

  term.start
  pause (5000)
  
  term.tx(0, CLS)                                    '                                                               ' time to open PST
  term.str(0, @RawInput)                                    '                                                               ' time to open PST
  count := 0
  term.str(0, string(13, "port 1 to port 2 to port 3 to port 1" ,13 ,13)) ' port connection sequence
repeat 
  repeat port from 1 to 3                               
    count++
    term.str(port, string("sent by port"))            ' send the text string to the designated port 
    term.decf(port, port, 2)                            ' send the port number to the designated port
    term.decf(port, count, 4)                           ' send the count to the designated port
    term.tx(port, 13)

  repeat port from 1 to 3
    case term.rxstr(port, @RawInput, 13)                ' rxstr (@receive buffer, string terminator) gets string
      0: term.str(0, string(" Normal return"))          
      -1: term.str(0, string(" Buffer overrun"))        ' send status of rx string to debug terminal

    term.str(0, string(" on port"))
    term.decf(0, port, 2)                               ' Print port number 
    term.str(0, string("  RECEIVED: "))
    term.str(0, @RawInput)                              ' print what was received
    term.tx(0, 13)                                
    bytefill(@RawInput, " ", 24)
    pause(5000)

pub pause(ms) | t
''*********************************************************************
'' Delay program in milliseconds
''*********************************************************************

  if (ms < 1)                                                    ' delay must be > 0
    return
  else
    t := cnt - 1776                                              ' sync with system counter
    repeat ms                                                    ' run delay
      waitcnt(t += MS_001)
   
dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}